/* out_ui.c *****************************************************************
 
 Twiddler Driver (userspace)
 -- UInput 'output' driver.
 
 Copyright (C) 2005, Chris Collins <chris@collins.id.au>
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author, Christopher Collins, nor any of his
    aliases may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY CHRISTOPHER COLLINS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL CHRISTOPHER COLLINS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <linux/input.h>
#include <linux/uinput.h>

#include "twiddled.h"

enum od_state_e {
  OD_STATE_INIT = 0,
  OD_STATE_RUNNING,
  OD_STATE_CLOSED
};

static int		myfd = -1;
static enum od_state_e	state = OD_STATE_CLOSED;

int
odev_open(char *devpath)
{
  struct uinput_user_dev	uidev;
  int ctr;

  atexit(odev_close);

  myfd = open(devpath, O_RDWR);
  if (myfd < 0) {
    perror("Couldn't open output event device");
    exit (1);
  }
  state = OD_STATE_INIT;

  memset(&uidev, 0, sizeof(uidev));
  strncpy(uidev.name, "Twiddler 1", UINPUT_MAX_NAME_SIZE);
  uidev.id.bustype = BUS_RS232;
  uidev.id.vendor = 0x0000;
  uidev.id.product = 0x0000;
  uidev.id.version = 0x100;
  /* set the masks */
  ioctl(myfd, UI_SET_EVBIT, EV_KEY);
  ioctl(myfd, UI_SET_EVBIT, EV_REL);
  ioctl(myfd, UI_SET_EVBIT, EV_REP);
  ioctl(myfd, UI_SET_RELBIT, REL_X);
  ioctl(myfd, UI_SET_RELBIT, REL_Y);
  /* enable all the keys we have in our keytable */
  for (ctr = 0; ctr <= keycount; ctr++) {
    ioctl(myfd, UI_SET_KEYBIT, keymap[ctr].scancode);
  }
  /* enable the mouse buttons */
  ioctl(myfd, UI_SET_KEYBIT, BTN_LEFT);
  ioctl(myfd, UI_SET_KEYBIT, BTN_RIGHT);
  ioctl(myfd, UI_SET_KEYBIT, BTN_MIDDLE);
  /* send the configuration buffer */
  write (myfd, &uidev, sizeof(uidev));

  /* start the input device */
  if (ioctl(myfd, UI_DEV_CREATE)) {
    perror ("failed to create uinput device");
    exit (1);
  }
  state = OD_STATE_RUNNING;

  return myfd;
}

void
odev_close()
{
  if (state == OD_STATE_CLOSED) {
    return;
  }
  if (state == OD_STATE_RUNNING) {
    ioctl(myfd, UI_DEV_DESTROY);
  }
  close(myfd);
  state = OD_STATE_CLOSED;
}

void
odev_sendkeydown(unsigned short scancode)
{
	struct input_event		e;
	memset(&e, 0, sizeof(e));
	
	e.type = EV_KEY;
	e.code = scancode;
	e.value = 1;
	write (myfd, &e, sizeof(e));
}

void
odev_sendkeyup(unsigned short scancode)
{
	struct input_event		e;
	memset(&e, 0, sizeof(e));
	
	e.type = EV_KEY;
	e.code = scancode;
	e.value = 0;
	write (myfd, &e, sizeof(e));
}
