/* twiddled.c  **************************************************************
 
 Twiddler Driver (userspace)
 
 Copyright (C) 2005, Chris Collins <chris@collins.id.au>
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author, Christopher Collins, nor any of his
    aliases may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY CHRISTOPHER COLLINS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL CHRISTOPHER COLLINS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "twiddled.h"

int		fd_serin = -1;
int		fd_eventout = -1;

struct thw_callback_s	nullcallback = {
	NULL,
	NULL,
	NULL,
};

int
main (int argc, char **argv)
{
	char	*serdev = "/dev/twidder";
	char	*evtdev = "/dev/input/uinput";
	char	*defmap = "/etc/twiddled/default.tmap";
	char	*options = "i:e:m:";
	int		i;
	
	i = getopt(argc, argv, options);
	while (i >= 0) {
	  	switch (i) {
	    case 'i':
	      serdev = strdup(optarg);
	      break;
	    case 'e':
	      evtdev = strdup(optarg);
	      break;
	    case 'm':
	      defmap = strdup(optarg);
	      break;
	    case '?':
	      exit (1);
	    default:
	      fprintf(stderr, "Unhandled option '%c'\n", optopt);
	      exit (1);
	    }
	    i = getopt(argc, argv, options);
	}

	/* initailise the state machine first */
	decode_init(defmap);
	
	fd_serin = thw_open(serdev);				   
	fd_eventout = odev_open(evtdev);
	/* nullcallback once to set up initial state */
	thw_update(&nullcallback);
	/* now run normally */
	while (1) {
	  thw_update(&decode_callback);
	}
	
	return 0;
}
